﻿
#region namespaces

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


using NUnit.Framework;

using Siemens.Automation.OMSPlus.Test.OMSp_Managed_HelpLib;


#endregion namespaces

namespace NunitTest
{
	/// <summary>
	/// base class for each test
	/// </summary>
	public class TestClsBase
	{

		#region variables

		// BaseFuncCls
		BaseFuncCls m_objDistribTest = new BaseFuncCls(); 

		// watchdog
		WatchDog m_WatchDog = null;

		// error 
		CheckErrors m_checkErrors = null;

		#endregion variables


		#region setup


		public void RunBeforeTestCase()
		{
			Console.WriteLine("RunBeforeTestCase(): start");

			///////////////////
			// create WatchDog
			/////////////////

			m_checkErrors = new CheckErrors();

			m_WatchDog = new WatchDog();
			m_WatchDog.OnStartTestEventHandler += new startTestEvent(startTest);
			m_WatchDog.OnWriteLogBeforeStop += new writeLogBeforeStop(writeErrorsInLog);
			
			m_objDistribTest.initBaseFuncCls(m_WatchDog, m_checkErrors);

			////////////////
			// other operations
			////////////////

			m_DistribTest.RunBeforeTestCase();

			Console.WriteLine("RunBeforeTestCase():end");
		}


		public void RunBeforeScenario()
		{
			m_DistribTest.RunBeforeScenario();
		}


		#endregion setup



		#region teardown


		public void RunAfterTestCase()
		{
			Console.WriteLine("RunAfterTestCase(): start");


			///////////////////
			// stop WatchDog
			/////////////////

			// holen den Stand
			bool bTestOk = m_WatchDog.TestSuccessful;

			// set Event to free - out test is OK
			m_WatchDog.setEvent();

			// stop events
			m_WatchDog.OnStartTestEventHandler -= new startTestEvent(startTest);
			m_WatchDog.OnWriteLogBeforeStop -= new writeLogBeforeStop(writeErrorsInLog);

			// stop WD
			m_WatchDog.stopWatchdog();
			m_WatchDog = null;

			Console.WriteLine("RunAfterTestCase(): WD = null");

			///////////////////
			// stop all
			///////////////////

			m_DistribTest.RunAfterTestCase();

			Console.WriteLine("RunAfterTestCase(): m_DistribTest.RunAfterTestCase() - ok");


			if (!bTestOk)
			{
				Assert.Fail("RunAfterTestCase(): TEST was STOPPED !!!!!!!!!!!!!!!!");
			}

			Console.WriteLine("RunAfterTestCase(): end");
		}


		public void RunAfterScenario()
		{
			Console.WriteLine("RunAfterScenario(): start");

			// if WD was not stopped - stop it
			if (m_WatchDog != null)
			{
				m_WatchDog.stopWatchdog();
				m_WatchDog = null;
			}

			m_DistribTest.RunAfterScenario();
		}


		#endregion teardown


		#region m_DistribTest, WatchDogForTest, CheckError

		/// <summary>
		/// get m_objDistribTest
		/// </summary>
		public BaseFuncCls m_DistribTest
		{
			get { return m_objDistribTest; }
		}


		/// <summary>
		/// get m_WatchDog
		/// </summary>
		public WatchDog WatchDogForTest
		{
			get { return m_WatchDog; }
		}


		/// <summary>
		/// get m_checkErrors
		/// </summary>
		public CheckErrors CheckError
		{
			get { return m_checkErrors; }
		}


		#endregion m_DistribTest, WatchDogForTest


		#region start / end test


		/// <summary>
		/// start one test
		/// </summary>
		/// <param name="sFuncName"></param>
		public void startCurrentTest(string sFuncName)
		{
			WatchDogForTest.setTestName(sFuncName);
			WatchDogForTest.startWatchdog();
		}


		/// <summary>
		/// start one Test
		/// </summary>
		/// <param name="sFuncName"></param>
		public void endCurrentTest(string sFuncName)
		{
			string msg = sFuncName + ":: test was successful()";
			Console.WriteLine(msg);

			// set Event to free - out test is OK
			WatchDogForTest.setEvent();
			msg = sFuncName + " setEvent() was ok";
			Console.WriteLine(msg);
		}


		/// <summary>
		/// write all Error-Messages in Log
		/// </summary>
		public void writeErrorsInLog()
		{
			if (m_checkErrors != null)
			{
				for (int i = 0; i < m_checkErrors.getErrorCount(); i++)
				{
					string sFuncName = ""; 
					string sErrorMessage = ""; 
					int nErrorCode = 0;
					Exception curException = null;

					m_checkErrors.getOneError(ref sFuncName, ref sErrorMessage, ref nErrorCode, ref curException, i);

					string msg = "All Errors *** " + sFuncName + "():";
					if (nErrorCode != 0)
					{
						msg += " error code = " + Convert.ToString(nErrorCode) + " :: " + sErrorMessage;
					}

					if (curException != null)
					{
						msg += "source = " + curException.Source + " :: " + curException.Message;
					}
					
					Console.WriteLine(msg);

				}	// for (int i = 0; i < m_checkErrors.getErrorCount(); i++)

			}	// if (m_checkErrors != null)

		}
		

		/// <summary>
		/// must be overriden in each test class
		/// </summary>
		/// <param name="sFuncName"></param>
		virtual public void startTest(string sFuncName)
		{
			return;
		}


		#endregion start / end test

	}
}
