﻿
#region namespaces

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;


using NUnit.Framework;

#endregion namespaces


namespace Siemens.Automation.OMSPlus.Test.OMSp_Managed_HelpLib
{

	#region delegates for events


	/// <summary>
	/// delegate - start test
	/// </summary>
	/// <param name="sParam"></param>
	public delegate void startTestEvent(string sParam);


	/// <summary>
	/// delegate - write log before stop of the test
	/// </summary>
	public delegate void writeLogBeforeStop();


	#endregion delegates for events



	/// <summary>
	/// watchdog-class
	/// </summary>
	public class WatchDog
	{

		#region const, variables

		// timeout for our event
		const int c_nEventWait = 2000;

		// event
		System.Threading.AutoResetEvent m_checkEvent = null;

		// name of the test
		string m_sTestName = "";

		// test thread
		System.Threading.Thread m_threadTest = null;

		// flag
		bool m_bWasTestOk = true;

		#endregion variables


		#region events

		// events
		public event startTestEvent OnStartTestEventHandler;
		public event writeLogBeforeStop OnWriteLogBeforeStop;

		#endregion event


		#region ctor

		/// <summary>
		/// ctor
		/// </summary>
		public WatchDog()
		{
			// event is busy
			m_checkEvent = new System.Threading.AutoResetEvent(false);	
			Assert.IsNotNull(m_checkEvent, "WatchDog(): m_checkEvent = NULL...");
		}


		#endregion ctor


		#region get / set

		/// <summary>
		/// get c_nEventWait
		/// </summary>
		public int EventTimeout
		{
			get { return c_nEventWait; }
		}

		/// <summary>
		/// return m_bWasTestOk: if m_bWasTestOk = true, the test was OK
		/// </summary>
		public bool TestSuccessful
		{
			get { return m_bWasTestOk; }
		}

		/// <summary>
		/// set the name of the test
		/// </summary>
		/// <param name="sFuncName"></param>
		public void setTestName(string sFuncName)
		{
			m_sTestName = sFuncName;
		}


		/// <summary>
		/// set event state to free
		/// </summary>
		/// <returns></returns>
		public bool setEvent()
		{
			// set event to state = free 
			bool bWasSet = m_checkEvent.Set();
			Assert.IsTrue(bWasSet, "setEvent(): Set() is failed...");
			return bWasSet;
		}


		#endregion get / set


		#region watchDog


		/// <summary>
		/// start of the test-thread
		/// </summary>
		public void startWatchdog()
		{
			Console.WriteLine("startWatchdog(): start");


			///////////////////
			// start test
			///////////////////

			// create object
			m_threadTest = new System.Threading.Thread(new System.Threading.ThreadStart(StartTest));
			Assert.IsNotNull(m_threadTest, "startWatchdog(): m_threadTest = NULL...");

			// start thread
			m_threadTest.Start();
			while (!m_threadTest.IsAlive) ;

			// WaitEvent
			WatchProcess();

			Console.WriteLine("startWatchdog(): end");

		}


		/// <summary>
		/// stop watchdog-thread
		/// </summary>
		public void stopWatchdog()
		{
			stopTest();
		}


		/// <summary>
		/// process function
		/// </summary>
		void WatchProcess()
		{
			bool bProcessOk = true;

			Assert.IsNotNull(m_checkEvent, "WatchProcess(): m_checkEvent = NULL...");
			Console.WriteLine("WatchProcess(): before WaitOne()");

			// check event (if free - change the event state to busy)
			bProcessOk = m_checkEvent.WaitOne(c_nEventWait, true);

			if (!bProcessOk)
			{
				// kill test
				cancelTest();
				return;
			}
			Console.WriteLine("WatchProcess(): after WaitOne(): bProcessOk = TRUE ");

		}


		#endregion watchDog


		#region test


		/// <summary>
		/// start the current test
		/// </summary>
		void StartTest()
		{
			Console.WriteLine("StartTest(): start");

			// start the test
			if (OnStartTestEventHandler != null)
			{
				OnStartTestEventHandler(m_sTestName);
			}

			Console.WriteLine("StartTest(): end");
		}

		/// <summary>
		/// stop the current test
		/// </summary>
		public void stopTest()
		{
			Console.WriteLine("stopTest(): start");

			if (m_threadTest != null)
			{
				Console.WriteLine("stopTest(): stop the thread");

				// write all errors in log
				if (OnWriteLogBeforeStop != null)
				{
					OnWriteLogBeforeStop();
					Thread.Sleep(10000);
				}

				// kill test-thread
				m_threadTest.Join();
				m_threadTest = null;
			}

			Console.WriteLine("stopTest(): end");
		}
	

		/// <summary>
		/// stop the test (main process)
		/// </summary>
		void cancelTest()
		{
			Console.WriteLine("cancelTest(): >>>>>>>>>>>>>>>  start");

			// stoppen test-thread
			stopTest();

			// set our flag
			m_bWasTestOk = false;

			Console.WriteLine("cancelTest(): >>>>>>>>>>>>>>>  end >>>>>>>>>>>>>>>>>>>>>");
		}
	

		#endregion test


	}
}
